require "util"
require "defines"
require "story"

game.on_init(function()
  game.disable_tips_and_tricks()
  local mapConf = game.map_settings
  mapConf.pollution.enabled = false
  mapConf.enemy_evolution.time_factor = 0
  mapConf.enemy_expansion.min_base_spacing = 0
  mapConf.enemy_expansion.min_player_base_distance = 5 --balance?
  
  global.market = game.get_entity_by_tag("market")
  global.grail1 = game.get_entity_by_tag("grail1")
  global.grail2 = game.get_entity_by_tag("grail2")
  
  global.market.add_market_item { price = {{"coin",  1}}, offer = {type = "give-item", item = "wood", count=10} }
  global.market.add_market_item { price = {{"coin", 10}}, offer = {type = "give-item", item = "raw-fish" } }
  global.market.add_market_item { price = {{"coin", 20}}, offer = {type = "give-item", item = "alien-artifact" } }
  global.market.add_market_item { price = {{"coin", 10}, {"advanced-circuit", 20}, {"processing-unit", 20}}, offer = { type = "give-item", item = "speed-module-3"} }
  global.market.add_market_item { price = {{"coin", 10}, {"advanced-circuit", 20}, {"processing-unit", 20}}, offer = { type = "give-item", item = "effectivity-module-3"} }

  local technologies = game.player.force.technologies
  technologies["automation"].researched = true
  technologies["logistics"].researched = true
  technologies["military"].researched = true
  technologies["military-2"].researched = true
  technologies["optics"].researched = true
  technologies["laser"].researched = true
  technologies["steel-processing"].researched = true
  technologies["automation-2"].researched = true
  technologies["turrets"].researched = true
  technologies["armor-making"].researched = true
  technologies["armor-making-2"].researched = true
  technologies["electric-energy-distribution-1"].researched = true
  technologies["advanced-material-processing"].enabled = true
  technologies["electronics"].researched = true
  technologies["advanced-electronics"].researched = true
  technologies["solar-energy"].researched = true
  technologies["inserter-stack-size-bonus-1"].researched = true
  technologies["inserter-stack-size-bonus-2"].researched = true

  --game.player.force.chart {{-500,-500},{1000,1000}}	
  initattackdata()
end)

-- Definition of the level behaviour, it is used by the story module
storytable =
{
  -- Functions that can be registered to check something regardless of the position in the story
  ["update-functions"]=
  {
  },
  -- The story contents
  {
    {
      action =
      function()
        game.show_message_dialog {text = {"msg-intro"}}
        game.show_message_dialog {text = {"msg-intro2"}}
        game.player.set_goal_description {"goal-prepare"}
      end
    },
    {
		condition = function(event)
			local player = game.player.position
			return (player.x > 71 and player.y < -40) or (player.y <-75 and player.x > 50)
		end,
		action = function() 
			game.player.set_goal_description {"goal-destroy", getSpawnerCount(), tostring(game.evolution_factor), global.attackdata.untilnextattack}
			--mainstory:addupdate("check-alive")
			global.rushStarted = true
		end
    },
    {
	  update = function()
        game.player.set_goal_description({"goal-destroy", getSpawnerCount(), tostring(game.evolution_factor), global.attackdata.untilnextattack}, true)
      end,
      condition = function()
		return getSpawnerCount()<=0
	  end,
      action = function()
        game.set_game_state {gamefinished=true, playerwon=true}
      end
    },
    
  }
}

mainstory = story.Story:new(storytable)

function difficultynumber(easy, normal, hard)
  if game.difficulty == defines.difficulty.easy then
    return easy
  elseif game.difficulty == defines.difficulty.normal then
    return normal
  else
    return hard
  end
end

initattackdata = function()
  global.attackdata = {
    -- volatile data
    attackcount = difficultynumber(10, 15, 20),
    untilnextattack = difficultynumber(3600 * 8, 3600 * 5, 3600 * 2),
    -- persistent data
    attackspacing = 3600 * 2,
    attackcountincrease = difficultynumber(3, 5, 10),
    initialevolution = difficultynumber(0.3, 0.4, 0.5),
  }
  global.evolution_factor = global.attackdata.initialevolution
end

function getSpawnerCount()
	return game.forces.enemy.get_entity_count("biter-spawner")
end

function handleattacks()
  global.attackdata.untilnextattack = global.attackdata.untilnextattack - 1
  if (global.grail1.valid or global.grail2.valid) and global.attackdata.untilnextattack <= 0 then
	local target = global.grail1
	if not target.valid or (global.grail2.valid and math.random()<=0.5) then
		target = global.grail2
	end
	
	local command = {
		type=defines.command.attack,
		target=game.player.character, --target,
		distraction=defines.distraction.none
	}
    global.group.set_command(command)
    game.player.print("sent " .. #global.group.members .. " units")
	
    if global.group.state == defines.groupstate.attacking then
        global.attackdata.attackcount = global.attackdata.attackcount + global.attackdata.attackcountincrease
        global.attackdata.untilnextattack = global.attackdata.attackspacing
        global.group = nil
    else
        game.player.print("not attacking")
	end
  elseif global.group == nil then
    global.group = createGroup(global.attackdata.attackcount)
  end 
end

function rewardKill(event)
	if event.entity.force~=game.player.force then
		local reward = 0
		if event.entity.name == "small-biter" then
			reward=1
		elseif event.entity.name == "medium-biter" then
			reward=2
		elseif event.entity.name == "big-biter" then
			reward=5
		elseif event.entity.name == "small-worm-turret" then
			reward=5
		elseif event.entity.name == "medium-worm-turret" then
			reward=10
		elseif event.entity.name == "big-worm-turret" then
			reward=20
		end
		if reward ~=0 then
			game.player.insert {name="coin", count=reward}
		end
	end
end

function createGroup(count)
    local list =  {first = 0, last = -1}

    function push(value)
      local last = list.last + 1
      list.last = last
      list[last] = value
    end
    
    function pop ()
      local first = list.first
      if empty() then error("list is empty") end
      local value = list[first]
      list[first] = nil -- to allow garbage collection
      list.first = first + 1
      return value
    end
    
    function empty()
        return list.first > list.last
    end

    local surface = game.get_surface(1)
    local group = surface.create_unit_group({position = { x=115, y=-100 }})
    local pos = 120
    
    push {{55,-120},{120,-45}}
    push {{0, -pos-10},{pos+10, -pos}}
    push {{pos, -pos}, {pos+10, 0}}
    pos = pos + 10
    
    while #group.members < count and pos < 500 do
        local units = surface.find_entities(pop())
        for i, enemy in ipairs(units) do
            if enemy.type=="unit" and enemy.force ==(game.forces.enemy) then
                group.add_member(enemy)
            end
            if #group.members >= count then
                break
            end
        end
        if empty then
            push {{0, -pos-10},{pos+10, -pos}}
            push {{pos, -pos}, {pos+10, 0}}
            pos = pos + 10
        end
    end
    
    return group
end

game.on_save(function()
  global.storydata = mainstory:export()
end)

game.on_load(function()
  mainstory:import(global.storydata)
end)

remote.add_interface("scenario", {
   createGroup = function(count)
        game.player.print(type(count))
        game.player.print(count)
        return createGroup(count)
   end
 })



game.on_event(defines.events, function(event)
  mainstory:update(event,"")
  if event.name == defines.events.on_tick and global.rushStarted then
	handleattacks()
  elseif event.name == defines.events.on_entity_died then
	rewardKill(event)
	if not (global.grail1.valid and global.grail2.valid) then
		game.set_game_state {gamefinished=true, playerwon=false}
	end
  end
end)

